# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

class Draggable(object):
    def dragText(self):
        return None

    def dragURL(self):
        return None

    def dragCustom(self):
        pass
